xe=0:0.01:7000;
ye=cos(xe)+cos(9*xe);
emd_visu(xe,ye,emd(xe,ye),1,'y=cos(9x)+cos(x) avec 700.000 points');
pause;

xe=0:0.01:10000;
ye=cos(xe/400)+cos(9*xe/400);
emd_visu(xe,ye,emd(xe,ye),1,'y=cos(9x/400)+cos(x/400) avec 1.000.000 points');
pause;

clear xe ye ze
close(1)
