xe=0:0.01:14.99;

%phenomene de Gibbs
ye=sin(xe)+sin(3*xe)/3+sin(5*xe)/5+sin(7*xe)/7+sin(9*xe)/9+sin(11*xe)/11+sin(13*xe)/13;
plot(xe,ye)
title('Phenomene de Gibbs')
pause

ye(1:300)=ones(1,300);
ye(301:600)=-ones(1,300);
ye(601:900)=ones(1,300);
ye(901:1200)=-ones(1,300);
ye(1201:1500)=ones(1,300);

%amplitudes égales,wsinus=w
figure(1)
ze=sin((pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{sinus}=\omega');
pause

%amplitudes égales,wsinus=3w
figure(1)
ze=sin(pi*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{sinus}=3\omega');
pause

%amplitudes égales,wsinus=5w
figure(1)
ze=sin((5*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{sinus}=5\omega');
pause

%amplitudes égales,wsinus=9w
figure(1)
ze=sin((9*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{sinus}=9\omega');
pause

%amplitudes égales,wsinus=11w
figure(1)
ze=sin((11*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{sinus}=11\omega');
pause


%amplitudes égales,wsinus=91w
figure(1)
ze=sin((91*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{sinus}=91\omega');
pause


clear xe ye ze
close(1)
close(2)