% Analyse un signal par EMD (Empirical Mode Decomposition)
% i.e. extraction d'IMFs (Intrinsic Mode Function) d'un signal
%
% Appel a emd :
%       imfs=emd(x,y,nb_imfs,epsilon);
%   ou  imfs=emd(x,y,epsilon,nb_imfs);
%
% Entrees :
%    x : vecteur des abscisses des echantillons du signal a analyser
%    y : vecteur des ordonnees des echantillons du signal a analyser
%    nb_imfs (optionnel) : nombre d'IMFs a extraire du signal
%      -> par defaut, le programme s'arrete lorsqu'il n'y a plus d'IMFs
%         a extraire du signal
%    epsilon (optionnel) : parametre de controle de la boucle interne du
%                          programme
%      -> par défaut, epsilon=0.001 (la moyenne d'un mode est considérée
%         comme nulle si son amplitude crete a crete est inferieure a :
%         epsilon*(amplitude crete a crete du signal y en entree); epsilon
%         appartient a ]0,0.1]
%
% Sorties :
%    imfs : matrice contenant les IMFs extraites (une IMF par ligne) ainsi
%           que le residu
%