xe=0:0.01:14.99;

ye(1:300)=-xe(1:300)+1.5;
ye(301:600)=xe(301:600)-4.5;
ye(601:900)=-xe(601:900)+7.5;
ye(901:1200)=xe(901:1200)-10.5;
ye(1201:1500)=-xe(1201:1500)+13.5;

%amplitudes égales,wcosinus=w
figure(1)
ze=1.5*cos((pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{cosinus}=\omega');
pause

%amplitudes égales,wtriangle=3w
figure(1)
ze=1.5*cos(pi*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{cosinus}=3\omega');
pause

%amplitudes égales,wcosinus=5w
figure(1)
ze=1.5*cos((5*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{cosinus}=5\omega');
pause

%amplitudes égales,wcosinus=7w
figure(1)
ze=1.5*cos((7*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{cosinus}=7\omega');
pause

%amplitudes égales,wcosinus=9w
figure(1)
ze=1.5*cos((9*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{cosinus}=9\omega');
pause

%amplitudes égales,wcosinus=11w
figure(1)
ze=1.5*cos((11*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{cosinus}=11\omega');
pause

%amplitudes égales,wcosinus=35w
figure(1)
ze=1.5*cos((35*pi/3)*xe);
subplot(2,1,1)
plot(xe,ye)
subplot(2,1,2)
plot(xe,ze)
emd_visu(xe,ye+ze,emd(xe,ye+ze),2,'amplitudes égales, \omega_{cosinus}=35\omega');
pause

clear xe ye ze
close(1)
close(2)