#ifndef __MIN_MAX_H__
#define __MIN_MAX_H__

/* structure dans laquelle on stocke les   */
/* listes de minima et de maxima du signal */
typedef struct {
  int n_min;
  int n_max;
  double *x_min;
  double *y_min;
  double *x_max;
  double *y_max;
} min_max_t;

min_max_t initialiser_min_max(int);
void min_max(double *,double *,int,min_max_t *);
void liberer_min_max(min_max_t);

#endif
